@@ -4,6 +4,7 @@ use strict;
 
 use File::Which;
 use File::Basename;
+use File::Path;
 use Getopt::Std;
 use HTML::Template;
 use Pod::Usage;
@@ -13,7 +14,7 @@ use CVS::Metrics;
 use Tk;
 
 my %opts;
-getopts('f:hst:vDHS:', \%opts);
+getopts('f:ho:st:vDHS:', \%opts);
 
 if ($opts{h}) {
 	pod2usage(-verbose => 1);
@@ -68,13 +69,19 @@ if ($opts{S}) {
 	$start_date = "2003/01/01" unless (defined $start_date);
 }
 
+my $output = $opts{o};
+if ($output and ! -d $output) {
+	mkpath $output
+			or die "can't create $output ($!).";
+}
+
 =head1 NAME
 
 cvs_tklog - Extract from cvs log
 
 =head1 SYNOPSIS
 
-cvs_tklog [B<-f> I<file.log>] [B<-t> I<title>] [B<-s>] [B<-D>] [B<-H>] [B<-S> I<yyyy/mm/dd>]
+cvs_tklog [B<-f> I<file.log>] [B<-o> I<dir>] [B<-t> I<title>] [B<-s>] [B<-D>] [B<-H>] [B<-S> I<yyyy/mm/dd>]
 
 =head1 OPTIONS
 
@@ -88,6 +95,10 @@ Mode off-line.
 
 Display Usage.
 
+=item -o
+
+Output directory.
+
 =item -s
 
 use an extern style sheet (cvs_tklog.css).
@@ -163,10 +174,10 @@ Francois PERRAD, francois.perrad@gadz.org
 
 =cut
 
-my $parser = new CVS::Metrics::Parser();
-if ($parser) {
-	my $cvs_log = $parser->parse($cvs_logfile);
-
+my $cvs_log = CVS::Metrics::CvsLog(
+		stream		=> $cvs_logfile,
+);
+if ($cvs_log) {
 	my @tags;
 	my $timed = $cvs_log->getTimedTag();
 	my %matched;
@@ -187,7 +198,7 @@ if ($parser) {
 		$cvs_log->insertHead();
 	}
 
-	my $top = new Top($cvs_log, \@tags, $title, $flg_dead, $flg_css, $start_date);
+	my $top = new Top($cvs_log, \@tags, $title, $flg_dead, $flg_css, $start_date, $output);
 	MainLoop();
 }
 
@@ -221,7 +232,7 @@ sub FindCvs {
 #######################################################################
 
 sub GenerateHTML {
-	my ($cvs_log, $tags, $title, $path, $tag_from, $tag_to, $flg_css, $start_date) = @_;
+	my ($cvs_log, $tags, $title, $path, $tag_from, $tag_to, $flg_css, $start_date, $output) = @_;
 
 my $html = q{
 <?xml version='1.0' encoding='ISO-8859-1'?>
@@ -347,16 +358,16 @@ my $style = q{
 	my $now = localtime();
 	my $generator = "cvs_tklog " . $CVS::Metrics::VERSION . " (Perl " . $] . ")";
 	my $dir = $path eq "." ? "all" : $path;
-	my $title_full = "${title}_${dir}_${tag_from}_to_${tag_to}";
-	$title_full =~ s/\//_/g;
+	my $title_full = "${title} ${dir} ${tag_from} to ${tag_to}";
 
 	my $image = $cvs_log->EnergyGD($tags, $path, $dir, 600, 400, $tag_from, $tag_to);
 
 	my $e_img = "e_${title_full}.png";
-	$e_img =~ s/\//_/g;
+	$e_img =~ s/[ \/]/_/g;
 	if (defined $image) {
-		open OUT, "> $e_img"
-				or die "can't open $e_img ($!).\n";
+		my $filename = (defined $output) ? $output . "/" . $e_img : $e_img;
+		open OUT, "> $filename"
+				or die "can't open $filename ($!).\n";
 		binmode OUT, ":raw";
 		print OUT $image->png();
 		close OUT;
@@ -383,10 +394,11 @@ my $style = q{
 	$image = $cvs_log->ActivityGD($path, $dir, $start_date, 800, 225, $date_from, $date_to);
 
 	my $a_img = "a_${title_full}.png";
-	$a_img =~ s/\//_/g;
+	$a_img =~ s/[ \/]/_/g;
 	if (defined $image) {
-		open OUT, "> $a_img"
-				or die "can't open $a_img ($!).\n";
+		my $filename = (defined $output) ? $output . "/" . $a_img : $a_img;
+		open OUT, "> $filename"
+				or die "can't open $filename ($!).\n";
 		binmode OUT, ":raw";
 		print OUT $image->png();
 		close OUT;
@@ -481,8 +493,9 @@ my $style = q{
 			dirs		=> \@dirs,
 	);
 
-	my $filename = "${title_full}.html";
-	$filename =~ s/\//_/g;
+	my $basename = "${title_full}.html";
+	$basename =~ s/[ \/]/_/g;
+	my $filename = (defined $output) ? $output . "/" . $basename : $basename;
 	open OUT, "> $filename"
 			or die "can't open $filename ($!)\n";
 	print OUT $template->output();
@@ -490,6 +503,8 @@ my $style = q{
 
 	if ($flg_css) {
 		my $stylesheet = "cvs_tklog.css";
+		$stylesheet = $output . "/" . $stylesheet
+				if ($output);
 		unless (-e $stylesheet) {
 			open OUT, "> $stylesheet"
 					or die "can't open $stylesheet ($!)\n";
@@ -512,12 +527,13 @@ sub new {
 	my $class = ref($proto) || $proto;
 	my $self = {};
 	bless($self, $class);
-	my ($cvs_log, $tags, $title, $flg_dead, $flg_css, $start_date) = @_;
+	my ($cvs_log, $tags, $title, $flg_dead, $flg_css, $start_date, $output) = @_;
 	$self->{cvs_log} = $cvs_log;
 	$self->{tags} = $tags;
 	$self->{title} = $title;
 	$self->{css} = $flg_css;
 	$self->{start_date} = $start_date;
+	$self->{output} = $output;
 
 	$self->{mw} = MainWindow->new();
 	$self->{mw}->title("CVS log - $title");
@@ -686,7 +702,7 @@ sub _audit {
 		$path =~ s/\./\//g;
 	}
 
-	my $html = main::GenerateHTML($self->{cvs_log}, $self->{tags}, $self->{title}, $path, $self->{tag_from}, $self->{tag_to}, $self->{css}, $self->{start_date});
+	my $html = main::GenerateHTML($self->{cvs_log}, $self->{tags}, $self->{title}, $path, $self->{tag_from}, $self->{tag_to}, $self->{css}, $self->{start_date}, $self->{output});
 	print "Starting browser...\n";
 	system "start $html";
 }
